import { RESTDataSource } from 'apollo-datasource-rest';
/** options for each client instance */
export interface BaseParams {
    params?: {
        query?: Record<string, unknown>;
    };
}
export type PathItemObject = {
    [M in HttpMethod]: OperationObject;
} & {
    parameters?: unknown;
};
export type ParseAs = 'json' | 'text' | 'blob' | 'arrayBuffer' | 'stream';
export interface OperationObject {
    parameters: unknown;
    requestBody: unknown;
    responses: unknown;
}
export type HttpMethod = 'get' | 'put' | 'post' | 'delete' | 'options' | 'head' | 'patch' | 'trace';
export type OkStatus = 200 | 201 | 202 | 203 | 204 | 206 | 207 | '2xx';
/** Get a union of paths which have method */
export type PathsWith<Paths extends Record<string, PathItemObject>, PathnameMethod extends HttpMethod> = {
    [Pathname in keyof Paths]: Paths[Pathname] extends {
        [K in PathnameMethod]: unknown;
    } ? Pathname : never;
}[keyof Paths];
/** Find first match of multiple keys */
export type FilterKeys<Obj, Matchers> = {
    [K in keyof Obj]: K extends Matchers ? Obj[K] : never;
}[keyof Obj];
export type MediaType = `${string}/${string}`;
export type Params<T> = T extends {
    parameters: unknown;
} ? {
    params: NonNullable<T['parameters']>;
} : BaseParams;
export type RequestBodyObj<T> = T extends {
    requestBody?: unknown;
} ? T['requestBody'] : never;
export type RequestBodyContent<T> = undefined extends RequestBodyObj<T> ? FilterKeys<NonNullable<RequestBodyObj<T>>, 'content'> | undefined : FilterKeys<RequestBodyObj<T>, 'content'>;
export type RequestBodyMedia<T> = FilterKeys<RequestBodyContent<T>, MediaType> extends never ? FilterKeys<NonNullable<RequestBodyContent<T>>, MediaType> | undefined : FilterKeys<RequestBodyContent<T>, MediaType>;
export type RequestBody<T> = RequestBodyMedia<T> extends never ? {
    body?: never;
} : undefined extends RequestBodyMedia<T> ? {
    body?: RequestBodyMedia<T>;
} : {
    body: RequestBodyMedia<T>;
};
export type RequestOptions<T> = Params<T> & RequestBody<T> & object;
export type Success<T> = FilterKeys<FilterKeys<T, OkStatus>, 'content'>;
type ApolloRequestInit = Parameters<RESTDataSource['get']>[2];
export type DataResponse<T> = T extends {
    responses: unknown;
} ? NonNullable<FilterKeys<Success<T['responses']>, MediaType>> : unknown | never;
export declare class TypedRESTDataSource<Paths extends {}, Context> extends RESTDataSource<Context> {
    protected openapi: {
        /** Call a GET endpoint */
        GET<P extends PathsWith<Paths, 'get'>>(url: P, init: RequestOptions<FilterKeys<Paths[P], 'get'>>, options?: ApolloRequestInit): Promise<DataResponse<'get' extends keyof Paths[P] ? Paths[P]['get'] : unknown>>;
        /** Call a PUT endpoint */
        PUT<P extends PathsWith<Paths, 'put'>>(url: P, init: RequestOptions<FilterKeys<Paths[P], 'put'>>, options?: ApolloRequestInit): Promise<DataResponse<'put' extends keyof Paths[P] ? Paths[P]['put'] : unknown>>;
        /** Call a POST endpoint */
        POST<P extends PathsWith<Paths, 'post'>>(url: P, init: RequestOptions<FilterKeys<Paths[P], 'post'>>, options?: ApolloRequestInit): Promise<DataResponse<'post' extends keyof Paths[P] ? Paths[P]['post'] : unknown>>;
        /** Call a DEL endpoint */
        DELETE<P extends PathsWith<Paths, 'delete'>>(url: P, init: RequestOptions<FilterKeys<Paths[P], 'delete'>>, options?: ApolloRequestInit): Promise<DataResponse<'delete' extends keyof Paths[P] ? Paths[P]['delete'] : unknown>>;
        /** Call a PATCH endpoint */
        PATCH<P extends PathsWith<Paths, 'patch'>>(url: P, init: RequestOptions<FilterKeys<Paths[P], 'patch'>>, options?: ApolloRequestInit): Promise<DataResponse<'patch' extends keyof Paths[P] ? Paths[P]['patch'] : unknown>>;
    };
    private setupParams;
    constructor(httpFetch?: ConstructorParameters<typeof RESTDataSource>[0]);
}
export {};
