"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.TypedRESTDataSource = void 0;
const apollo_datasource_rest_1 = require("apollo-datasource-rest");
function buildFinalPath(url, params) {
    let finalPath = url;
    if (params?.path) {
        for (const [k, v] of Object.entries(params.path)) {
            finalPath = finalPath.replace(`{${k}}`, encodeURIComponent(String(v)));
        }
    }
    return {
        path: finalPath,
    };
}
class TypedRESTDataSource extends apollo_datasource_rest_1.RESTDataSource {
    openapi;
    setupParams(hasBody, url, init, options) {
        const baseParams = init;
        let apolloOptions = options ? { ...options } : undefined;
        const { path } = buildFinalPath(url, baseParams?.params);
        const query = baseParams?.params?.query;
        if (hasBody) {
            if (query) {
                // Looks like apollo-datasource-rest doesn't support query params with some methods.
                // So we will pipe them through, knowing where it looks for them. We could
                // instead append them to the URL, but then that's redoing query serialization
                apolloOptions = apolloOptions || {};
                apolloOptions.params = query;
            }
            const body = init?.body;
            if (apolloOptions) {
                return [path, body, apolloOptions];
            }
            if (body) {
                return [path, body];
            }
            return [path];
        }
        if (apolloOptions) {
            return [path, query, apolloOptions];
        }
        if (query) {
            return [path, query];
        }
        return [path];
    }
    constructor(httpFetch) {
        super(httpFetch);
        this.openapi = {
            GET: async (url, init, options) => {
                const params = this.setupParams(false, url, init, options);
                return this.get(...params);
            },
            DELETE: async (url, init, options) => {
                const params = this.setupParams(false, url, init, options);
                return this.delete(...params);
            },
            PUT: async (url, init, options) => {
                const params = this.setupParams(true, url, init, options);
                return this.put(...params);
            },
            POST: async (url, init, options) => {
                const params = this.setupParams(true, url, init, options);
                return this.post(...params);
            },
            PATCH: async (url, init, options) => {
                const params = this.setupParams(true, url, init, options);
                return this.patch(...params);
            },
        };
    }
}
exports.TypedRESTDataSource = TypedRESTDataSource;
//# sourceMappingURL=client.js.map